#!/usr/bin/env python
##################################################
# File Name: __init__.py
# Author: Rui
# mail: rdong1989@gmail.com
# Created Time: Tue 23 Jul 2019 12:36:49 PM EDT
################################################

import sys

__author__ = ("Rui Dong")
__email__ = ("rdong1989@gmail.com")

from . import gini
from . import fano
from . import consensus
from . import plot
